# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import Dict, List, Optional
from datetime import date, datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.currency import Currency
from .reconciliation_rule import ReconciliationRule
from .expected_payment_type import ExpectedPaymentType

__all__ = ["ExpectedPayment"]


class ExpectedPayment(BaseModel):
    id: str

    amount_lower_bound: Optional[int] = None
    """The lowest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_reconciled: Optional[int] = None
    """The amount reconciled for this expected payment.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_reconciled_direction: Optional[Literal["credit", "debit"]] = None
    """One of credit or debit.

    Indicates whether amount_reconciled is a credit or debit amount.
    """

    amount_unreconciled: Optional[int] = None
    """The amount that remains unreconciled for this expected payment.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    amount_unreconciled_direction: Optional[Literal["credit", "debit"]] = None
    """One of credit or debit.

    Indicates whether amount_unreconciled is a credit or debit amount.
    """

    amount_upper_bound: Optional[int] = None
    """The highest amount this expected payment may be equal to.

    Value in specified currency's smallest unit. e.g. $10 would be represented
    as 1000.
    """

    counterparty_id: Optional[str] = None
    """The ID of the counterparty you expect for this payment."""

    created_at: datetime

    currency: Optional[Currency] = None
    """Must conform to ISO 4217. Defaults to the currency of the internal account."""

    date_lower_bound: Optional[date] = None
    """The earliest date the payment may come in. Format: yyyy-mm-dd"""

    date_upper_bound: Optional[date] = None
    """The latest date the payment may come in. Format: yyyy-mm-dd"""

    description: Optional[str] = None
    """An optional description for internal use."""

    direction: Optional[Literal["credit", "debit"]] = None
    """One of credit or debit.

    When you are receiving money, use credit. When you are being charged, use debit.
    """

    external_id: Optional[str] = None
    """An optional user-defined 180 character unique identifier."""

    internal_account_id: Optional[str] = None
    """The ID of the Internal Account for the expected payment."""

    ledger_transaction_id: Optional[str] = None
    """The ID of the ledger transaction linked to the expected payment."""

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    object: str

    reconciliation_filters: Optional[builtins.object] = None
    """The reconciliation filters you have for this payment."""

    reconciliation_groups: Optional[builtins.object] = None
    """The reconciliation groups you have for this payment."""

    reconciliation_method: Optional[Literal["automatic", "manual"]] = None
    """
    One of manual if this expected payment was manually reconciled in the dashboard,
    automatic if it was automatically reconciled by Modern Treasury, or null if it
    is unreconciled.
    """

    reconciliation_rule_variables: Optional[List[ReconciliationRule]] = None
    """An array of reconciliation rule variables for this payment."""

    remittance_information: Optional[str] = None
    """For `ach`, this field will be passed through on an addenda record.

    For `wire` payments the field will be passed through as the "Originator to
    Beneficiary Information", also known as OBI or Fedwire tag 6000.
    """

    statement_descriptor: Optional[str] = None
    """The statement description you expect to see on the transaction.

    For ACH payments, this will be the full line item passed from the bank. For wire
    payments, this will be the OBI field on the wire. For check payments, this will
    be the memo field.
    """

    status: Literal["archived", "partially_reconciled", "reconciled", "unreconciled"]
    """One of unreconciled, partially_reconciled, reconciled, or archived."""

    transaction_id: Optional[str] = None
    """The ID of the Transaction this expected payment object has been matched to."""

    transaction_line_item_id: Optional[str] = None
    """The ID of the Transaction Line Item this expected payment has been matched to."""

    type: Optional[ExpectedPaymentType] = None
    """
    One of: ach, au_becs, bacs, book, check, eft, interac, provxchange, rtp, sen,
    sepa, signet, wire.
    """

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["BulkResultListParams"]


class BulkResultListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    entity_id: str
    """Unique identifier for the result entity object."""

    entity_type: Literal[
        "payment_order",
        "ledger_account",
        "ledger_transaction",
        "expected_payment",
        "transaction",
        "entity_link",
        "transaction_line_item",
        "bulk_error",
    ]
    """The type of the request that created this result.

    bulk_request is the only supported `request_type`
    """

    per_page: int

    request_id: str
    """Unique identifier for the request that created this bulk result.

    This is the ID of the bulk request when `request_type` is bulk_request
    """

    request_type: Literal["bulk_request"]
    """The type of the request that created this result.

    bulk_request is the only supported `request_type`
    """

    status: Literal["pending", "successful", "failed"]
    """One of successful or failed."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .account_entry_delete_params import AccountEntryDeleteParams as AccountEntryDeleteParams
from .account_entry_update_params import AccountEntryUpdateParams as AccountEntryUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["HoldUpdateParams"]


class HoldUpdateParams(TypedDict, total=False):
    status: Required[Literal["resolved"]]
    """The new status of the hold"""

    resolution: Optional[str]
    """The resolution of the hold"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from modern_treasury import ModernTreasury, AsyncModernTreasury
from modern_treasury.types import (
    VirtualAccount,
)
from modern_treasury.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestVirtualAccounts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            account_details=[
                {
                    "account_number": "account_number",
                    "account_number_type": "au_number",
                }
            ],
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            credit_ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            debit_ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            ledger_account={
                "currency": "currency",
                "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "normal_balance": "credit",
                "currency_exponent": 0,
                "description": "description",
                "external_id": "external_id",
                "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "ledgerable_type": "counterparty",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
            },
            metadata={"foo": "string"},
            routing_details=[
                {
                    "routing_number": "routing_number",
                    "routing_number_type": "aba",
                    "payment_type": "ach",
                }
            ],
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: ModernTreasury) -> None:
        response = client.virtual_accounts.with_raw_response.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: ModernTreasury) -> None:
        with client.virtual_accounts.with_streaming_response.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.retrieve(
            "id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.virtual_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.virtual_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_accounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.update(
            id="id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.update(
            id="id",
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: ModernTreasury) -> None:
        response = client.virtual_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: ModernTreasury) -> None:
        with client.virtual_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_accounts.with_raw_response.update(
                id="",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.list()
        assert_matches_type(SyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.list(
            after_cursor="after_cursor",
            counterparty_id="counterparty_id",
            internal_account_id="internal_account_id",
            metadata={"foo": "string"},
            per_page=0,
        )
        assert_matches_type(SyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.virtual_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(SyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.virtual_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(SyncPage[VirtualAccount], virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: ModernTreasury) -> None:
        virtual_account = client.virtual_accounts.delete(
            "id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: ModernTreasury) -> None:
        response = client.virtual_accounts.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: ModernTreasury) -> None:
        with client.virtual_accounts.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.virtual_accounts.with_raw_response.delete(
                "",
            )


class TestAsyncVirtualAccounts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
            account_details=[
                {
                    "account_number": "account_number",
                    "account_number_type": "au_number",
                }
            ],
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            credit_ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            debit_ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            description="description",
            ledger_account={
                "currency": "currency",
                "ledger_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "name": "name",
                "normal_balance": "credit",
                "currency_exponent": 0,
                "description": "description",
                "external_id": "external_id",
                "ledger_account_category_ids": ["182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e"],
                "ledgerable_id": "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                "ledgerable_type": "counterparty",
                "metadata": {
                    "key": "value",
                    "foo": "bar",
                    "modern": "treasury",
                },
            },
            metadata={"foo": "string"},
            routing_details=[
                {
                    "routing_number": "routing_number",
                    "routing_number_type": "aba",
                    "payment_type": "ach",
                }
            ],
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.virtual_accounts.with_raw_response.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.virtual_accounts.with_streaming_response.create(
            internal_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.retrieve(
            "id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.virtual_accounts.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.virtual_accounts.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_accounts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.update(
            id="id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.update(
            id="id",
            counterparty_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            ledger_account_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            metadata={"foo": "string"},
            name="name",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.virtual_accounts.with_raw_response.update(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.virtual_accounts.with_streaming_response.update(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_accounts.with_raw_response.update(
                id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.list()
        assert_matches_type(AsyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.list(
            after_cursor="after_cursor",
            counterparty_id="counterparty_id",
            internal_account_id="internal_account_id",
            metadata={"foo": "string"},
            per_page=0,
        )
        assert_matches_type(AsyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.virtual_accounts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(AsyncPage[VirtualAccount], virtual_account, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.virtual_accounts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(AsyncPage[VirtualAccount], virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncModernTreasury) -> None:
        virtual_account = await async_client.virtual_accounts.delete(
            "id",
        )
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.virtual_accounts.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        virtual_account = response.parse()
        assert_matches_type(VirtualAccount, virtual_account, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.virtual_accounts.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            virtual_account = await response.parse()
            assert_matches_type(VirtualAccount, virtual_account, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.virtual_accounts.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .versions import (
    Versions,
    AsyncVersions,
    VersionsWithRawResponse,
    AsyncVersionsWithRawResponse,
    VersionsWithStreamingResponse,
    AsyncVersionsWithStreamingResponse,
)
from .ledger_transactions import (
    LedgerTransactions,
    AsyncLedgerTransactions,
    LedgerTransactionsWithRawResponse,
    AsyncLedgerTransactionsWithRawResponse,
    LedgerTransactionsWithStreamingResponse,
    AsyncLedgerTransactionsWithStreamingResponse,
)

__all__ = [
    "Versions",
    "AsyncVersions",
    "VersionsWithRawResponse",
    "AsyncVersionsWithRawResponse",
    "VersionsWithStreamingResponse",
    "AsyncVersionsWithStreamingResponse",
    "LedgerTransactions",
    "AsyncLedgerTransactions",
    "LedgerTransactionsWithRawResponse",
    "AsyncLedgerTransactionsWithRawResponse",
    "LedgerTransactionsWithStreamingResponse",
    "AsyncLedgerTransactionsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ... import _legacy_response
from ..._types import Body, Query, Headers, NoneType, NotGiven, SequenceNotStr, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..._base_client import make_request_options
from ...types.ledger_account_settlements import account_entry_delete_params, account_entry_update_params

__all__ = ["AccountEntries", "AsyncAccountEntries"]


class AccountEntries(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountEntriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AccountEntriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountEntriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AccountEntriesWithStreamingResponse(self)

    def update(
        self,
        id: str,
        *,
        ledger_entry_ids: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add ledger entries to a draft ledger account settlement.

        Args:
          ledger_entry_ids: The ids of the ledger entries that are to be added or removed from the ledger
              account settlement.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            path_template("/api/ledger_account_settlements/{id}/ledger_entries", id=id),
            body=maybe_transform(
                {"ledger_entry_ids": ledger_entry_ids}, account_entry_update_params.AccountEntryUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    def delete(
        self,
        id: str,
        *,
        ledger_entry_ids: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Remove ledger entries from a draft ledger account settlement.

        Args:
          ledger_entry_ids: The ids of the ledger entries that are to be added or removed from the ledger
              account settlement.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/api/ledger_account_settlements/{id}/ledger_entries", id=id),
            body=maybe_transform(
                {"ledger_entry_ids": ledger_entry_ids}, account_entry_delete_params.AccountEntryDeleteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class AsyncAccountEntries(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountEntriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountEntriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountEntriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncAccountEntriesWithStreamingResponse(self)

    async def update(
        self,
        id: str,
        *,
        ledger_entry_ids: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Add ledger entries to a draft ledger account settlement.

        Args:
          ledger_entry_ids: The ids of the ledger entries that are to be added or removed from the ledger
              account settlement.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            path_template("/api/ledger_account_settlements/{id}/ledger_entries", id=id),
            body=await async_maybe_transform(
                {"ledger_entry_ids": ledger_entry_ids}, account_entry_update_params.AccountEntryUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )

    async def delete(
        self,
        id: str,
        *,
        ledger_entry_ids: Optional[SequenceNotStr[str]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Remove ledger entries from a draft ledger account settlement.

        Args:
          ledger_entry_ids: The ids of the ledger entries that are to be added or removed from the ledger
              account settlement.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/api/ledger_account_settlements/{id}/ledger_entries", id=id),
            body=await async_maybe_transform(
                {"ledger_entry_ids": ledger_entry_ids}, account_entry_delete_params.AccountEntryDeleteParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class AccountEntriesWithRawResponse:
    def __init__(self, account_entries: AccountEntries) -> None:
        self._account_entries = account_entries

        self.update = _legacy_response.to_raw_response_wrapper(
            account_entries.update,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            account_entries.delete,
        )


class AsyncAccountEntriesWithRawResponse:
    def __init__(self, account_entries: AsyncAccountEntries) -> None:
        self._account_entries = account_entries

        self.update = _legacy_response.async_to_raw_response_wrapper(
            account_entries.update,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            account_entries.delete,
        )


class AccountEntriesWithStreamingResponse:
    def __init__(self, account_entries: AccountEntries) -> None:
        self._account_entries = account_entries

        self.update = to_streamed_response_wrapper(
            account_entries.update,
        )
        self.delete = to_streamed_response_wrapper(
            account_entries.delete,
        )


class AsyncAccountEntriesWithStreamingResponse:
    def __init__(self, account_entries: AsyncAccountEntries) -> None:
        self._account_entries = account_entries

        self.update = async_to_streamed_response_wrapper(
            account_entries.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            account_entries.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["HoldCreateParams"]


class HoldCreateParams(TypedDict, total=False):
    status: Required[Literal["active"]]
    """The status of the hold"""

    target_id: Required[str]
    """The ID of the target to hold"""

    target_type: Required[Literal["payment_order"]]
    """The type of target to hold"""

    metadata: Optional[Dict[str, str]]
    """Additional metadata for the hold"""

    reason: Optional[str]
    """The reason for the hold"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .shared.transaction_direction import TransactionDirection

__all__ = ["IncomingPaymentDetailListParams"]


class IncomingPaymentDetailListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    as_of_date_end: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    Filters incoming payment details with an as_of_date starting on or before the
    specified date (YYYY-MM-DD).
    """

    as_of_date_start: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    Filters incoming payment details with an as_of_date starting on or after the
    specified date (YYYY-MM-DD).
    """

    direction: TransactionDirection
    """One of `credit` or `debit`."""

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    status: Literal["completed", "pending", "returned"]
    """The current status of the incoming payment order.

    One of `pending`, `completed`, or `returned`.
    """

    type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "check",
        "eft",
        "ethereum",
        "interac",
        "neft",
        "nz_becs",
        "polygon",
        "rtp",
        "sepa",
        "signet",
        "solana",
        "wire",
        "zengin",
    ]
    """
    One of: `ach`, `book`, `check`, `eft`, `interac`, `rtp`, `sepa`, `signet`, or
    `wire`.
    """

    virtual_account_id: str
    """
    If the incoming payment detail is in a virtual account, the ID of the Virtual
    Account.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["DocumentListParams"]


class DocumentListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    documentable_id: str
    """The unique identifier for the associated object."""

    documentable_type: Literal[
        "connections",
        "counterparties",
        "expected_payments",
        "external_accounts",
        "identifications",
        "incoming_payment_details",
        "internal_accounts",
        "legal_entities",
        "organizations",
        "payment_orders",
        "transactions",
    ]
    """The type of the associated object.

    Currently can be one of `payment_order`, `transaction`, `expected_payment`,
    `counterparty`, `organization`, `case`, `internal_account`, `decision`, or
    `external_account`.
    """

    per_page: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["LedgerBalance"]


class LedgerBalance(BaseModel):
    amount: int

    credits: int

    currency: str
    """The currency of the ledger account."""

    currency_exponent: int
    """The currency exponent of the ledger account."""

    debits: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from modern_treasury import ModernTreasury, AsyncModernTreasury

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestJournalEntries:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: ModernTreasury) -> None:
        journal_entry = client.journal_entries.retrieve(
            "id",
        )
        assert journal_entry is None

    @parametrize
    def test_raw_response_retrieve(self, client: ModernTreasury) -> None:
        response = client.journal_entries.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        journal_entry = response.parse()
        assert journal_entry is None

    @parametrize
    def test_streaming_response_retrieve(self, client: ModernTreasury) -> None:
        with client.journal_entries.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            journal_entry = response.parse()
            assert journal_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: ModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.journal_entries.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: ModernTreasury) -> None:
        journal_entry = client.journal_entries.list(
            journal_report_id="journal_report_id",
        )
        assert journal_entry is None

    @parametrize
    def test_method_list_with_all_params(self, client: ModernTreasury) -> None:
        journal_entry = client.journal_entries.list(
            journal_report_id="journal_report_id",
            page=0,
            per_page=0,
        )
        assert journal_entry is None

    @parametrize
    def test_raw_response_list(self, client: ModernTreasury) -> None:
        response = client.journal_entries.with_raw_response.list(
            journal_report_id="journal_report_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        journal_entry = response.parse()
        assert journal_entry is None

    @parametrize
    def test_streaming_response_list(self, client: ModernTreasury) -> None:
        with client.journal_entries.with_streaming_response.list(
            journal_report_id="journal_report_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            journal_entry = response.parse()
            assert journal_entry is None

        assert cast(Any, response.is_closed) is True


class TestAsyncJournalEntries:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncModernTreasury) -> None:
        journal_entry = await async_client.journal_entries.retrieve(
            "id",
        )
        assert journal_entry is None

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.journal_entries.with_raw_response.retrieve(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        journal_entry = response.parse()
        assert journal_entry is None

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.journal_entries.with_streaming_response.retrieve(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            journal_entry = await response.parse()
            assert journal_entry is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncModernTreasury) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.journal_entries.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncModernTreasury) -> None:
        journal_entry = await async_client.journal_entries.list(
            journal_report_id="journal_report_id",
        )
        assert journal_entry is None

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncModernTreasury) -> None:
        journal_entry = await async_client.journal_entries.list(
            journal_report_id="journal_report_id",
            page=0,
            per_page=0,
        )
        assert journal_entry is None

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncModernTreasury) -> None:
        response = await async_client.journal_entries.with_raw_response.list(
            journal_report_id="journal_report_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        journal_entry = response.parse()
        assert journal_entry is None

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncModernTreasury) -> None:
        async with async_client.journal_entries.with_streaming_response.list(
            journal_report_id="journal_report_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            journal_entry = await response.parse()
            assert journal_entry is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ContactDetail"]


class ContactDetail(TypedDict, total=False):
    id: Required[str]

    contact_identifier: Required[str]

    contact_identifier_type: Required[Literal["email", "phone_number", "website"]]

    created_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    discarded_at: Required[Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]]

    live_mode: Required[bool]
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: Required[str]

    updated_at: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import hold_list_params, hold_create_params, hold_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.hold_list_response import HoldListResponse
from ..types.hold_create_response import HoldCreateResponse
from ..types.hold_update_response import HoldUpdateResponse
from ..types.hold_retrieve_response import HoldRetrieveResponse

__all__ = ["Holds", "AsyncHolds"]


class Holds(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> HoldsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return HoldsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> HoldsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return HoldsWithStreamingResponse(self)

    def create(
        self,
        *,
        status: Literal["active"],
        target_id: str,
        target_type: Literal["payment_order"],
        metadata: Optional[Dict[str, str]] | Omit = omit,
        reason: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> HoldCreateResponse:
        """
        Create a new hold

        Args:
          status: The status of the hold

          target_id: The ID of the target to hold

          target_type: The type of target to hold

          metadata: Additional metadata for the hold

          reason: The reason for the hold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/holds",
            body=maybe_transform(
                {
                    "status": status,
                    "target_id": target_id,
                    "target_type": target_type,
                    "metadata": metadata,
                    "reason": reason,
                },
                hold_create_params.HoldCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=HoldCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HoldRetrieveResponse:
        """
        Get a specific hold

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/api/holds/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HoldRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        status: Literal["resolved"],
        resolution: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> HoldUpdateResponse:
        """
        Update a hold

        Args:
          status: The new status of the hold

          resolution: The resolution of the hold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            path_template("/api/holds/{id}", id=id),
            body=maybe_transform(
                {
                    "status": status,
                    "resolution": resolution,
                },
                hold_update_params.HoldUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=HoldUpdateResponse,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        status: Optional[Literal["active", "resolved"]] | Omit = omit,
        target_id: Optional[str] | Omit = omit,
        target_type: Optional[Literal["payment_order"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[HoldListResponse]:
        """
        Get a list of holds.

        Args:
          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          status: Translation missing: en.openapi.descriptions.payment_order.query_params.status

          target_id:
              Translation missing:
              en.openapi.descriptions.payment_order.query_params.target_id

          target_type:
              Translation missing:
              en.openapi.descriptions.payment_order.query_params.target_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/holds",
            page=SyncPage[HoldListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "metadata": metadata,
                        "per_page": per_page,
                        "status": status,
                        "target_id": target_id,
                        "target_type": target_type,
                    },
                    hold_list_params.HoldListParams,
                ),
            ),
            model=HoldListResponse,
        )


class AsyncHolds(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncHoldsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncHoldsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncHoldsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncHoldsWithStreamingResponse(self)

    async def create(
        self,
        *,
        status: Literal["active"],
        target_id: str,
        target_type: Literal["payment_order"],
        metadata: Optional[Dict[str, str]] | Omit = omit,
        reason: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> HoldCreateResponse:
        """
        Create a new hold

        Args:
          status: The status of the hold

          target_id: The ID of the target to hold

          target_type: The type of target to hold

          metadata: Additional metadata for the hold

          reason: The reason for the hold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/holds",
            body=await async_maybe_transform(
                {
                    "status": status,
                    "target_id": target_id,
                    "target_type": target_type,
                    "metadata": metadata,
                    "reason": reason,
                },
                hold_create_params.HoldCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=HoldCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> HoldRetrieveResponse:
        """
        Get a specific hold

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/api/holds/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=HoldRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        status: Literal["resolved"],
        resolution: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> HoldUpdateResponse:
        """
        Update a hold

        Args:
          status: The new status of the hold

          resolution: The resolution of the hold

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            path_template("/api/holds/{id}", id=id),
            body=await async_maybe_transform(
                {
                    "status": status,
                    "resolution": resolution,
                },
                hold_update_params.HoldUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=HoldUpdateResponse,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        status: Optional[Literal["active", "resolved"]] | Omit = omit,
        target_id: Optional[str] | Omit = omit,
        target_type: Optional[Literal["payment_order"]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[HoldListResponse, AsyncPage[HoldListResponse]]:
        """
        Get a list of holds.

        Args:
          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          status: Translation missing: en.openapi.descriptions.payment_order.query_params.status

          target_id:
              Translation missing:
              en.openapi.descriptions.payment_order.query_params.target_id

          target_type:
              Translation missing:
              en.openapi.descriptions.payment_order.query_params.target_type

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/holds",
            page=AsyncPage[HoldListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "metadata": metadata,
                        "per_page": per_page,
                        "status": status,
                        "target_id": target_id,
                        "target_type": target_type,
                    },
                    hold_list_params.HoldListParams,
                ),
            ),
            model=HoldListResponse,
        )


class HoldsWithRawResponse:
    def __init__(self, holds: Holds) -> None:
        self._holds = holds

        self.create = _legacy_response.to_raw_response_wrapper(
            holds.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            holds.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            holds.update,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            holds.list,
        )


class AsyncHoldsWithRawResponse:
    def __init__(self, holds: AsyncHolds) -> None:
        self._holds = holds

        self.create = _legacy_response.async_to_raw_response_wrapper(
            holds.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            holds.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            holds.update,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            holds.list,
        )


class HoldsWithStreamingResponse:
    def __init__(self, holds: Holds) -> None:
        self._holds = holds

        self.create = to_streamed_response_wrapper(
            holds.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            holds.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            holds.update,
        )
        self.list = to_streamed_response_wrapper(
            holds.list,
        )


class AsyncHoldsWithStreamingResponse:
    def __init__(self, holds: AsyncHolds) -> None:
        self._holds = holds

        self.create = async_to_streamed_response_wrapper(
            holds.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            holds.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            holds.update,
        )
        self.list = async_to_streamed_response_wrapper(
            holds.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Optional
from datetime import date, datetime

import httpx

from .. import _legacy_response
from ..types import foreign_exchange_quote_list_params, foreign_exchange_quote_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.shared.currency import Currency
from ..types.foreign_exchange_quote import ForeignExchangeQuote

__all__ = ["ForeignExchangeQuotes", "AsyncForeignExchangeQuotes"]


class ForeignExchangeQuotes(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ForeignExchangeQuotesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return ForeignExchangeQuotesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ForeignExchangeQuotesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return ForeignExchangeQuotesWithStreamingResponse(self)

    def create(
        self,
        *,
        internal_account_id: str,
        target_currency: Currency,
        base_amount: int | Omit = omit,
        base_currency: Currency | Omit = omit,
        effective_at: Union[str, datetime] | Omit = omit,
        target_amount: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> ForeignExchangeQuote:
        """
        create foreign_exchange_quote

        Args:
          internal_account_id: The ID for the `InternalAccount` this quote is associated with.

          target_currency: Currency to convert the `base_currency` to, often called the "buy" currency.

          base_amount: Amount in the lowest denomination of the `base_currency` to convert, often
              called the "sell" amount.

          base_currency: Currency to convert, often called the "sell" currency.

          effective_at: The timestamp until when the quoted rate is valid.

          target_amount: Amount in the lowest denomination of the `target_currency`, often called the
              "buy" amount.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return self._post(
            "/api/foreign_exchange_quotes",
            body=maybe_transform(
                {
                    "internal_account_id": internal_account_id,
                    "target_currency": target_currency,
                    "base_amount": base_amount,
                    "base_currency": base_currency,
                    "effective_at": effective_at,
                    "target_amount": target_amount,
                },
                foreign_exchange_quote_create_params.ForeignExchangeQuoteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ForeignExchangeQuote,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForeignExchangeQuote:
        """
        get foreign_exchange_quote

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/api/foreign_exchange_quotes/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForeignExchangeQuote,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        base_currency: str | Omit = omit,
        effective_at_end: Union[str, date] | Omit = omit,
        effective_at_start: Union[str, date] | Omit = omit,
        expires_at: Union[str, datetime] | Omit = omit,
        internal_account_id: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        target_currency: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[ForeignExchangeQuote]:
        """
        list foreign_exchange_quotes

        Args:
          base_currency: Currency to convert, often called the "sell" currency.

          effective_at_end: An inclusive upper bound for searching effective_at

          effective_at_start: An inclusive lower bound for searching effective_at

          expires_at: The timestamp until which the quote must be booked by.

          internal_account_id: The ID for the `InternalAccount` this quote is associated with.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          target_currency: Currency to convert the `base_currency` to, often called the "buy" currency.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/foreign_exchange_quotes",
            page=SyncPage[ForeignExchangeQuote],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "base_currency": base_currency,
                        "effective_at_end": effective_at_end,
                        "effective_at_start": effective_at_start,
                        "expires_at": expires_at,
                        "internal_account_id": internal_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                        "target_currency": target_currency,
                    },
                    foreign_exchange_quote_list_params.ForeignExchangeQuoteListParams,
                ),
            ),
            model=ForeignExchangeQuote,
        )


class AsyncForeignExchangeQuotes(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncForeignExchangeQuotesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncForeignExchangeQuotesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncForeignExchangeQuotesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncForeignExchangeQuotesWithStreamingResponse(self)

    async def create(
        self,
        *,
        internal_account_id: str,
        target_currency: Currency,
        base_amount: int | Omit = omit,
        base_currency: Currency | Omit = omit,
        effective_at: Union[str, datetime] | Omit = omit,
        target_amount: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> ForeignExchangeQuote:
        """
        create foreign_exchange_quote

        Args:
          internal_account_id: The ID for the `InternalAccount` this quote is associated with.

          target_currency: Currency to convert the `base_currency` to, often called the "buy" currency.

          base_amount: Amount in the lowest denomination of the `base_currency` to convert, often
              called the "sell" amount.

          base_currency: Currency to convert, often called the "sell" currency.

          effective_at: The timestamp until when the quoted rate is valid.

          target_amount: Amount in the lowest denomination of the `target_currency`, often called the
              "buy" amount.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        return await self._post(
            "/api/foreign_exchange_quotes",
            body=await async_maybe_transform(
                {
                    "internal_account_id": internal_account_id,
                    "target_currency": target_currency,
                    "base_amount": base_amount,
                    "base_currency": base_currency,
                    "effective_at": effective_at,
                    "target_amount": target_amount,
                },
                foreign_exchange_quote_create_params.ForeignExchangeQuoteCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=ForeignExchangeQuote,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ForeignExchangeQuote:
        """
        get foreign_exchange_quote

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/api/foreign_exchange_quotes/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ForeignExchangeQuote,
        )

    def list(
        self,
        *,
        after_cursor: Optional[str] | Omit = omit,
        base_currency: str | Omit = omit,
        effective_at_end: Union[str, date] | Omit = omit,
        effective_at_start: Union[str, date] | Omit = omit,
        expires_at: Union[str, datetime] | Omit = omit,
        internal_account_id: str | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        per_page: int | Omit = omit,
        target_currency: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ForeignExchangeQuote, AsyncPage[ForeignExchangeQuote]]:
        """
        list foreign_exchange_quotes

        Args:
          base_currency: Currency to convert, often called the "sell" currency.

          effective_at_end: An inclusive upper bound for searching effective_at

          effective_at_start: An inclusive lower bound for searching effective_at

          expires_at: The timestamp until which the quote must be booked by.

          internal_account_id: The ID for the `InternalAccount` this quote is associated with.

          metadata: For example, if you want to query for records with metadata key `Type` and value
              `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
              parameters.

          target_currency: Currency to convert the `base_currency` to, often called the "buy" currency.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/foreign_exchange_quotes",
            page=AsyncPage[ForeignExchangeQuote],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "base_currency": base_currency,
                        "effective_at_end": effective_at_end,
                        "effective_at_start": effective_at_start,
                        "expires_at": expires_at,
                        "internal_account_id": internal_account_id,
                        "metadata": metadata,
                        "per_page": per_page,
                        "target_currency": target_currency,
                    },
                    foreign_exchange_quote_list_params.ForeignExchangeQuoteListParams,
                ),
            ),
            model=ForeignExchangeQuote,
        )


class ForeignExchangeQuotesWithRawResponse:
    def __init__(self, foreign_exchange_quotes: ForeignExchangeQuotes) -> None:
        self._foreign_exchange_quotes = foreign_exchange_quotes

        self.create = _legacy_response.to_raw_response_wrapper(
            foreign_exchange_quotes.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            foreign_exchange_quotes.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            foreign_exchange_quotes.list,
        )


class AsyncForeignExchangeQuotesWithRawResponse:
    def __init__(self, foreign_exchange_quotes: AsyncForeignExchangeQuotes) -> None:
        self._foreign_exchange_quotes = foreign_exchange_quotes

        self.create = _legacy_response.async_to_raw_response_wrapper(
            foreign_exchange_quotes.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            foreign_exchange_quotes.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            foreign_exchange_quotes.list,
        )


class ForeignExchangeQuotesWithStreamingResponse:
    def __init__(self, foreign_exchange_quotes: ForeignExchangeQuotes) -> None:
        self._foreign_exchange_quotes = foreign_exchange_quotes

        self.create = to_streamed_response_wrapper(
            foreign_exchange_quotes.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            foreign_exchange_quotes.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            foreign_exchange_quotes.list,
        )


class AsyncForeignExchangeQuotesWithStreamingResponse:
    def __init__(self, foreign_exchange_quotes: AsyncForeignExchangeQuotes) -> None:
        self._foreign_exchange_quotes = foreign_exchange_quotes

        self.create = async_to_streamed_response_wrapper(
            foreign_exchange_quotes.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            foreign_exchange_quotes.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            foreign_exchange_quotes.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .holds import (
    Holds,
    AsyncHolds,
    HoldsWithRawResponse,
    AsyncHoldsWithRawResponse,
    HoldsWithStreamingResponse,
    AsyncHoldsWithStreamingResponse,
)
from .events import (
    Events,
    AsyncEvents,
    EventsWithRawResponse,
    AsyncEventsWithRawResponse,
    EventsWithStreamingResponse,
    AsyncEventsWithStreamingResponse,
)
from .ledgers import (
    Ledgers,
    AsyncLedgers,
    LedgersWithRawResponse,
    AsyncLedgersWithRawResponse,
    LedgersWithStreamingResponse,
    AsyncLedgersWithStreamingResponse,
)
from .returns import (
    Returns,
    AsyncReturns,
    ReturnsWithRawResponse,
    AsyncReturnsWithRawResponse,
    ReturnsWithStreamingResponse,
    AsyncReturnsWithStreamingResponse,
)
from .invoices import (
    Invoices,
    AsyncInvoices,
    InvoicesWithRawResponse,
    AsyncInvoicesWithRawResponse,
    InvoicesWithStreamingResponse,
    AsyncInvoicesWithStreamingResponse,
)
from .webhooks import Webhooks, AsyncWebhooks
from .documents import (
    Documents,
    AsyncDocuments,
    DocumentsWithRawResponse,
    AsyncDocumentsWithRawResponse,
    DocumentsWithStreamingResponse,
    AsyncDocumentsWithStreamingResponse,
)
from .line_items import (
    LineItems,
    AsyncLineItems,
    LineItemsWithRawResponse,
    AsyncLineItemsWithRawResponse,
    LineItemsWithStreamingResponse,
    AsyncLineItemsWithStreamingResponse,
)
from .connections import (
    Connections,
    AsyncConnections,
    ConnectionsWithRawResponse,
    AsyncConnectionsWithRawResponse,
    ConnectionsWithStreamingResponse,
    AsyncConnectionsWithStreamingResponse,
)
from .validations import (
    Validations,
    AsyncValidations,
    ValidationsWithRawResponse,
    AsyncValidationsWithRawResponse,
    ValidationsWithStreamingResponse,
    AsyncValidationsWithStreamingResponse,
)
from .bulk_results import (
    BulkResults,
    AsyncBulkResults,
    BulkResultsWithRawResponse,
    AsyncBulkResultsWithRawResponse,
    BulkResultsWithStreamingResponse,
    AsyncBulkResultsWithStreamingResponse,
)
from .transactions import (
    Transactions,
    AsyncTransactions,
    TransactionsWithRawResponse,
    AsyncTransactionsWithRawResponse,
    TransactionsWithStreamingResponse,
    AsyncTransactionsWithStreamingResponse,
)
from .bulk_requests import (
    BulkRequests,
    AsyncBulkRequests,
    BulkRequestsWithRawResponse,
    AsyncBulkRequestsWithRawResponse,
    BulkRequestsWithStreamingResponse,
    AsyncBulkRequestsWithStreamingResponse,
)
from .payment_flows import (
    PaymentFlows,
    AsyncPaymentFlows,
    PaymentFlowsWithRawResponse,
    AsyncPaymentFlowsWithRawResponse,
    PaymentFlowsWithStreamingResponse,
    AsyncPaymentFlowsWithStreamingResponse,
)
from .counterparties import (
    Counterparties,
    AsyncCounterparties,
    CounterpartiesWithRawResponse,
    AsyncCounterpartiesWithRawResponse,
    CounterpartiesWithStreamingResponse,
    AsyncCounterpartiesWithStreamingResponse,
)
from .ledger_entries import (
    LedgerEntries,
    AsyncLedgerEntries,
    LedgerEntriesWithRawResponse,
    AsyncLedgerEntriesWithRawResponse,
    LedgerEntriesWithStreamingResponse,
    AsyncLedgerEntriesWithStreamingResponse,
)
from .legal_entities import (
    LegalEntities,
    AsyncLegalEntities,
    LegalEntitiesWithRawResponse,
    AsyncLegalEntitiesWithRawResponse,
    LegalEntitiesWithStreamingResponse,
    AsyncLegalEntitiesWithStreamingResponse,
)
from .payment_orders import (
    PaymentOrders,
    AsyncPaymentOrders,
    PaymentOrdersWithRawResponse,
    AsyncPaymentOrdersWithRawResponse,
    PaymentOrdersWithStreamingResponse,
    AsyncPaymentOrdersWithStreamingResponse,
)
from .account_details import (
    AccountDetails,
    AsyncAccountDetails,
    AccountDetailsWithRawResponse,
    AsyncAccountDetailsWithRawResponse,
    AccountDetailsWithStreamingResponse,
    AsyncAccountDetailsWithStreamingResponse,
)
from .journal_entries import (
    JournalEntries,
    AsyncJournalEntries,
    JournalEntriesWithRawResponse,
    AsyncJournalEntriesWithRawResponse,
    JournalEntriesWithStreamingResponse,
    AsyncJournalEntriesWithStreamingResponse,
)
from .journal_reports import (
    JournalReports,
    AsyncJournalReports,
    JournalReportsWithRawResponse,
    AsyncJournalReportsWithRawResponse,
    JournalReportsWithStreamingResponse,
    AsyncJournalReportsWithStreamingResponse,
)
from .ledger_accounts import (
    LedgerAccounts,
    AsyncLedgerAccounts,
    LedgerAccountsWithRawResponse,
    AsyncLedgerAccountsWithRawResponse,
    LedgerAccountsWithStreamingResponse,
    AsyncLedgerAccountsWithStreamingResponse,
)
from .payment_actions import (
    PaymentActions,
    AsyncPaymentActions,
    PaymentActionsWithRawResponse,
    AsyncPaymentActionsWithRawResponse,
    PaymentActionsWithStreamingResponse,
    AsyncPaymentActionsWithStreamingResponse,
)
from .routing_details import (
    RoutingDetails,
    AsyncRoutingDetails,
    RoutingDetailsWithRawResponse,
    AsyncRoutingDetailsWithRawResponse,
    RoutingDetailsWithStreamingResponse,
    AsyncRoutingDetailsWithStreamingResponse,
)
from .virtual_accounts import (
    VirtualAccounts,
    AsyncVirtualAccounts,
    VirtualAccountsWithRawResponse,
    AsyncVirtualAccountsWithRawResponse,
    VirtualAccountsWithStreamingResponse,
    AsyncVirtualAccountsWithStreamingResponse,
)
from .expected_payments import (
    ExpectedPayments,
    AsyncExpectedPayments,
    ExpectedPaymentsWithRawResponse,
    AsyncExpectedPaymentsWithRawResponse,
    ExpectedPaymentsWithStreamingResponse,
    AsyncExpectedPaymentsWithStreamingResponse,
)
from .external_accounts import (
    ExternalAccounts,
    AsyncExternalAccounts,
    ExternalAccountsWithRawResponse,
    AsyncExternalAccountsWithRawResponse,
    ExternalAccountsWithStreamingResponse,
    AsyncExternalAccountsWithStreamingResponse,
)
from .internal_accounts import (
    InternalAccounts,
    AsyncInternalAccounts,
    InternalAccountsWithRawResponse,
    AsyncInternalAccountsWithRawResponse,
    InternalAccountsWithStreamingResponse,
    AsyncInternalAccountsWithStreamingResponse,
)
from .payment_references import (
    PaymentReferences,
    AsyncPaymentReferences,
    PaymentReferencesWithRawResponse,
    AsyncPaymentReferencesWithRawResponse,
    PaymentReferencesWithStreamingResponse,
    AsyncPaymentReferencesWithStreamingResponse,
)
from .ledger_transactions import (
    LedgerTransactions,
    AsyncLedgerTransactions,
    LedgerTransactionsWithRawResponse,
    AsyncLedgerTransactionsWithRawResponse,
    LedgerTransactionsWithStreamingResponse,
    AsyncLedgerTransactionsWithStreamingResponse,
)
from .foreign_exchange_quotes import (
    ForeignExchangeQuotes,
    AsyncForeignExchangeQuotes,
    ForeignExchangeQuotesWithRawResponse,
    AsyncForeignExchangeQuotesWithRawResponse,
    ForeignExchangeQuotesWithStreamingResponse,
    AsyncForeignExchangeQuotesWithStreamingResponse,
)
from .account_collection_flows import (
    AccountCollectionFlows,
    AsyncAccountCollectionFlows,
    AccountCollectionFlowsWithRawResponse,
    AsyncAccountCollectionFlowsWithRawResponse,
    AccountCollectionFlowsWithStreamingResponse,
    AsyncAccountCollectionFlowsWithStreamingResponse,
)
from .incoming_payment_details import (
    IncomingPaymentDetails,
    AsyncIncomingPaymentDetails,
    IncomingPaymentDetailsWithRawResponse,
    AsyncIncomingPaymentDetailsWithRawResponse,
    IncomingPaymentDetailsWithStreamingResponse,
    AsyncIncomingPaymentDetailsWithStreamingResponse,
)
from .connection_legal_entities import (
    ConnectionLegalEntities,
    AsyncConnectionLegalEntities,
    ConnectionLegalEntitiesWithRawResponse,
    AsyncConnectionLegalEntitiesWithRawResponse,
    ConnectionLegalEntitiesWithStreamingResponse,
    AsyncConnectionLegalEntitiesWithStreamingResponse,
)
from .ledger_account_categories import (
    LedgerAccountCategories,
    AsyncLedgerAccountCategories,
    LedgerAccountCategoriesWithRawResponse,
    AsyncLedgerAccountCategoriesWithRawResponse,
    LedgerAccountCategoriesWithStreamingResponse,
    AsyncLedgerAccountCategoriesWithStreamingResponse,
)
from .ledger_account_statements import (
    LedgerAccountStatements,
    AsyncLedgerAccountStatements,
    LedgerAccountStatementsWithRawResponse,
    AsyncLedgerAccountStatementsWithRawResponse,
    LedgerAccountStatementsWithStreamingResponse,
    AsyncLedgerAccountStatementsWithStreamingResponse,
)
from .ledger_account_settlements import (
    LedgerAccountSettlements,
    AsyncLedgerAccountSettlements,
    LedgerAccountSettlementsWithRawResponse,
    AsyncLedgerAccountSettlementsWithRawResponse,
    LedgerAccountSettlementsWithStreamingResponse,
    AsyncLedgerAccountSettlementsWithStreamingResponse,
)
from .ledger_account_balance_monitors import (
    LedgerAccountBalanceMonitors,
    AsyncLedgerAccountBalanceMonitors,
    LedgerAccountBalanceMonitorsWithRawResponse,
    AsyncLedgerAccountBalanceMonitorsWithRawResponse,
    LedgerAccountBalanceMonitorsWithStreamingResponse,
    AsyncLedgerAccountBalanceMonitorsWithStreamingResponse,
)

__all__ = [
    "Connections",
    "AsyncConnections",
    "ConnectionsWithRawResponse",
    "AsyncConnectionsWithRawResponse",
    "ConnectionsWithStreamingResponse",
    "AsyncConnectionsWithStreamingResponse",
    "Counterparties",
    "AsyncCounterparties",
    "CounterpartiesWithRawResponse",
    "AsyncCounterpartiesWithRawResponse",
    "CounterpartiesWithStreamingResponse",
    "AsyncCounterpartiesWithStreamingResponse",
    "Events",
    "AsyncEvents",
    "EventsWithRawResponse",
    "AsyncEventsWithRawResponse",
    "EventsWithStreamingResponse",
    "AsyncEventsWithStreamingResponse",
    "ExpectedPayments",
    "AsyncExpectedPayments",
    "ExpectedPaymentsWithRawResponse",
    "AsyncExpectedPaymentsWithRawResponse",
    "ExpectedPaymentsWithStreamingResponse",
    "AsyncExpectedPaymentsWithStreamingResponse",
    "ExternalAccounts",
    "AsyncExternalAccounts",
    "ExternalAccountsWithRawResponse",
    "AsyncExternalAccountsWithRawResponse",
    "ExternalAccountsWithStreamingResponse",
    "AsyncExternalAccountsWithStreamingResponse",
    "IncomingPaymentDetails",
    "AsyncIncomingPaymentDetails",
    "IncomingPaymentDetailsWithRawResponse",
    "AsyncIncomingPaymentDetailsWithRawResponse",
    "IncomingPaymentDetailsWithStreamingResponse",
    "AsyncIncomingPaymentDetailsWithStreamingResponse",
    "Invoices",
    "AsyncInvoices",
    "InvoicesWithRawResponse",
    "AsyncInvoicesWithRawResponse",
    "InvoicesWithStreamingResponse",
    "AsyncInvoicesWithStreamingResponse",
    "Documents",
    "AsyncDocuments",
    "DocumentsWithRawResponse",
    "AsyncDocumentsWithRawResponse",
    "DocumentsWithStreamingResponse",
    "AsyncDocumentsWithStreamingResponse",
    "AccountCollectionFlows",
    "AsyncAccountCollectionFlows",
    "AccountCollectionFlowsWithRawResponse",
    "AsyncAccountCollectionFlowsWithRawResponse",
    "AccountCollectionFlowsWithStreamingResponse",
    "AsyncAccountCollectionFlowsWithStreamingResponse",
    "AccountDetails",
    "AsyncAccountDetails",
    "AccountDetailsWithRawResponse",
    "AsyncAccountDetailsWithRawResponse",
    "AccountDetailsWithStreamingResponse",
    "AsyncAccountDetailsWithStreamingResponse",
    "RoutingDetails",
    "AsyncRoutingDetails",
    "RoutingDetailsWithRawResponse",
    "AsyncRoutingDetailsWithRawResponse",
    "RoutingDetailsWithStreamingResponse",
    "AsyncRoutingDetailsWithStreamingResponse",
    "InternalAccounts",
    "AsyncInternalAccounts",
    "InternalAccountsWithRawResponse",
    "AsyncInternalAccountsWithRawResponse",
    "InternalAccountsWithStreamingResponse",
    "AsyncInternalAccountsWithStreamingResponse",
    "Ledgers",
    "AsyncLedgers",
    "LedgersWithRawResponse",
    "AsyncLedgersWithRawResponse",
    "LedgersWithStreamingResponse",
    "AsyncLedgersWithStreamingResponse",
    "LedgerAccountCategories",
    "AsyncLedgerAccountCategories",
    "LedgerAccountCategoriesWithRawResponse",
    "AsyncLedgerAccountCategoriesWithRawResponse",
    "LedgerAccountCategoriesWithStreamingResponse",
    "AsyncLedgerAccountCategoriesWithStreamingResponse",
    "LedgerAccounts",
    "AsyncLedgerAccounts",
    "LedgerAccountsWithRawResponse",
    "AsyncLedgerAccountsWithRawResponse",
    "LedgerAccountsWithStreamingResponse",
    "AsyncLedgerAccountsWithStreamingResponse",
    "LedgerAccountBalanceMonitors",
    "AsyncLedgerAccountBalanceMonitors",
    "LedgerAccountBalanceMonitorsWithRawResponse",
    "AsyncLedgerAccountBalanceMonitorsWithRawResponse",
    "LedgerAccountBalanceMonitorsWithStreamingResponse",
    "AsyncLedgerAccountBalanceMonitorsWithStreamingResponse",
    "LedgerAccountStatements",
    "AsyncLedgerAccountStatements",
    "LedgerAccountStatementsWithRawResponse",
    "AsyncLedgerAccountStatementsWithRawResponse",
    "LedgerAccountStatementsWithStreamingResponse",
    "AsyncLedgerAccountStatementsWithStreamingResponse",
    "LedgerEntries",
    "AsyncLedgerEntries",
    "LedgerEntriesWithRawResponse",
    "AsyncLedgerEntriesWithRawResponse",
    "LedgerEntriesWithStreamingResponse",
    "AsyncLedgerEntriesWithStreamingResponse",
    "LedgerTransactions",
    "AsyncLedgerTransactions",
    "LedgerTransactionsWithRawResponse",
    "AsyncLedgerTransactionsWithRawResponse",
    "LedgerTransactionsWithStreamingResponse",
    "AsyncLedgerTransactionsWithStreamingResponse",
    "LineItems",
    "AsyncLineItems",
    "LineItemsWithRawResponse",
    "AsyncLineItemsWithRawResponse",
    "LineItemsWithStreamingResponse",
    "AsyncLineItemsWithStreamingResponse",
    "PaymentFlows",
    "AsyncPaymentFlows",
    "PaymentFlowsWithRawResponse",
    "AsyncPaymentFlowsWithRawResponse",
    "PaymentFlowsWithStreamingResponse",
    "AsyncPaymentFlowsWithStreamingResponse",
    "PaymentOrders",
    "AsyncPaymentOrders",
    "PaymentOrdersWithRawResponse",
    "AsyncPaymentOrdersWithRawResponse",
    "PaymentOrdersWithStreamingResponse",
    "AsyncPaymentOrdersWithStreamingResponse",
    "PaymentReferences",
    "AsyncPaymentReferences",
    "PaymentReferencesWithRawResponse",
    "AsyncPaymentReferencesWithRawResponse",
    "PaymentReferencesWithStreamingResponse",
    "AsyncPaymentReferencesWithStreamingResponse",
    "Returns",
    "AsyncReturns",
    "ReturnsWithRawResponse",
    "AsyncReturnsWithRawResponse",
    "ReturnsWithStreamingResponse",
    "AsyncReturnsWithStreamingResponse",
    "Transactions",
    "AsyncTransactions",
    "TransactionsWithRawResponse",
    "AsyncTransactionsWithRawResponse",
    "TransactionsWithStreamingResponse",
    "AsyncTransactionsWithStreamingResponse",
    "Validations",
    "AsyncValidations",
    "ValidationsWithRawResponse",
    "AsyncValidationsWithRawResponse",
    "ValidationsWithStreamingResponse",
    "AsyncValidationsWithStreamingResponse",
    "Webhooks",
    "AsyncWebhooks",
    "VirtualAccounts",
    "AsyncVirtualAccounts",
    "VirtualAccountsWithRawResponse",
    "AsyncVirtualAccountsWithRawResponse",
    "VirtualAccountsWithStreamingResponse",
    "AsyncVirtualAccountsWithStreamingResponse",
    "BulkRequests",
    "AsyncBulkRequests",
    "BulkRequestsWithRawResponse",
    "AsyncBulkRequestsWithRawResponse",
    "BulkRequestsWithStreamingResponse",
    "AsyncBulkRequestsWithStreamingResponse",
    "BulkResults",
    "AsyncBulkResults",
    "BulkResultsWithRawResponse",
    "AsyncBulkResultsWithRawResponse",
    "BulkResultsWithStreamingResponse",
    "AsyncBulkResultsWithStreamingResponse",
    "LedgerAccountSettlements",
    "AsyncLedgerAccountSettlements",
    "LedgerAccountSettlementsWithRawResponse",
    "AsyncLedgerAccountSettlementsWithRawResponse",
    "LedgerAccountSettlementsWithStreamingResponse",
    "AsyncLedgerAccountSettlementsWithStreamingResponse",
    "ForeignExchangeQuotes",
    "AsyncForeignExchangeQuotes",
    "ForeignExchangeQuotesWithRawResponse",
    "AsyncForeignExchangeQuotesWithRawResponse",
    "ForeignExchangeQuotesWithStreamingResponse",
    "AsyncForeignExchangeQuotesWithStreamingResponse",
    "ConnectionLegalEntities",
    "AsyncConnectionLegalEntities",
    "ConnectionLegalEntitiesWithRawResponse",
    "AsyncConnectionLegalEntitiesWithRawResponse",
    "ConnectionLegalEntitiesWithStreamingResponse",
    "AsyncConnectionLegalEntitiesWithStreamingResponse",
    "LegalEntities",
    "AsyncLegalEntities",
    "LegalEntitiesWithRawResponse",
    "AsyncLegalEntitiesWithRawResponse",
    "LegalEntitiesWithStreamingResponse",
    "AsyncLegalEntitiesWithStreamingResponse",
    "PaymentActions",
    "AsyncPaymentActions",
    "PaymentActionsWithRawResponse",
    "AsyncPaymentActionsWithRawResponse",
    "PaymentActionsWithStreamingResponse",
    "AsyncPaymentActionsWithStreamingResponse",
    "JournalEntries",
    "AsyncJournalEntries",
    "JournalEntriesWithRawResponse",
    "AsyncJournalEntriesWithRawResponse",
    "JournalEntriesWithStreamingResponse",
    "AsyncJournalEntriesWithStreamingResponse",
    "JournalReports",
    "AsyncJournalReports",
    "JournalReportsWithRawResponse",
    "AsyncJournalReportsWithRawResponse",
    "JournalReportsWithStreamingResponse",
    "AsyncJournalReportsWithStreamingResponse",
    "Holds",
    "AsyncHolds",
    "HoldsWithRawResponse",
    "AsyncHoldsWithRawResponse",
    "HoldsWithStreamingResponse",
    "AsyncHoldsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, TypedDict

__all__ = ["HoldListParams"]


class HoldListParams(TypedDict, total=False):
    after_cursor: Optional[str]

    metadata: Dict[str, str]
    """
    For example, if you want to query for records with metadata key `Type` and value
    `Loan`, the query would be `metadata%5BType%5D=Loan`. This encodes the query
    parameters.
    """

    per_page: int

    status: Optional[Literal["active", "resolved"]]
    """Translation missing: en.openapi.descriptions.payment_order.query_params.status"""

    target_id: Optional[str]
    """
    Translation missing:
    en.openapi.descriptions.payment_order.query_params.target_id
    """

    target_type: Optional[Literal["payment_order"]]
    """
    Translation missing:
    en.openapi.descriptions.payment_order.query_params.target_type
    """



from __future__ import annotations

import re
from typing import (
    Any,
    Mapping,
    Callable,
)
from urllib.parse import quote

# Matches '.' or '..' where each dot is either literal or percent-encoded (%2e / %2E).
_DOT_SEGMENT_RE = re.compile(r"^(?:\.|%2[eE]){1,2}$")

_PLACEHOLDER_RE = re.compile(r"\{(\w+)\}")


def _quote_path_segment_part(value: str) -> str:
    """Percent-encode `value` for use in a URI path segment.

    Considers characters not in `pchar` set from RFC 3986 §3.3 to be unsafe.
    https://datatracker.ietf.org/doc/html/rfc3986#section-3.3
    """
    # quote() already treats unreserved characters (letters, digits, and -._~)
    # as safe, so we only need to add sub-delims, ':', and '@'.
    # Notably, unlike the default `safe` for quote(), / is unsafe and must be quoted.
    return quote(value, safe="!$&'()*+,;=:@")


def _quote_query_part(value: str) -> str:
    """Percent-encode `value` for use in a URI query string.

    Considers &, = and characters not in `query` set from RFC 3986 §3.4 to be unsafe.
    https://datatracker.ietf.org/doc/html/rfc3986#section-3.4
    """
    return quote(value, safe="!$'()*+,;:@/?")


def _quote_fragment_part(value: str) -> str:
    """Percent-encode `value` for use in a URI fragment.

    Considers characters not in `fragment` set from RFC 3986 §3.5 to be unsafe.
    https://datatracker.ietf.org/doc/html/rfc3986#section-3.5
    """
    return quote(value, safe="!$&'()*+,;=:@/?")


def _interpolate(
    template: str,
    values: Mapping[str, Any],
    quoter: Callable[[str], str],
) -> str:
    """Replace {name} placeholders in `template`, quoting each value with `quoter`.

    Placeholder names are looked up in `values`.

    Raises:
        KeyError: If a placeholder is not found in `values`.
    """
    # re.split with a capturing group returns alternating
    # [text, name, text, name, ..., text] elements.
    parts = _PLACEHOLDER_RE.split(template)

    for i in range(1, len(parts), 2):
        name = parts[i]
        if name not in values:
            raise KeyError(f"a value for placeholder {{{name}}} was not provided")
        val = values[name]
        if val is None:
            parts[i] = "null"
        elif isinstance(val, bool):
            parts[i] = "true" if val else "false"
        else:
            parts[i] = quoter(str(values[name]))

    return "".join(parts)


def path_template(template: str, /, **kwargs: Any) -> str:
    """Interpolate {name} placeholders in `template` from keyword arguments.

    Args:
        template: The template string containing {name} placeholders.
        **kwargs: Keyword arguments to interpolate into the template.

    Returns:
        The template with placeholders interpolated and percent-encoded.

        Safe characters for percent-encoding are dependent on the URI component.
        Placeholders in path and fragment portions are percent-encoded where the `segment`
        and `fragment` sets from RFC 3986 respectively are considered safe.
        Placeholders in the query portion are percent-encoded where the `query` set from
        RFC 3986 §3.3 is considered safe except for = and & characters.

    Raises:
        KeyError: If a placeholder is not found in `kwargs`.
        ValueError: If resulting path contains /./ or /../ segments (including percent-encoded dot-segments).
    """
    # Split the template into path, query, and fragment portions.
    fragment_template: str | None = None
    query_template: str | None = None

    rest = template
    if "#" in rest:
        rest, fragment_template = rest.split("#", 1)
    if "?" in rest:
        rest, query_template = rest.split("?", 1)
    path_template = rest

    # Interpolate each portion with the appropriate quoting rules.
    path_result = _interpolate(path_template, kwargs, _quote_path_segment_part)

    # Reject dot-segments (. and ..) in the final assembled path.  The check
    # runs after interpolation so that adjacent placeholders or a mix of static
    # text and placeholders that together form a dot-segment are caught.
    # Also reject percent-encoded dot-segments to protect against incorrectly
    # implemented normalization in servers/proxies.
    for segment in path_result.split("/"):
        if _DOT_SEGMENT_RE.match(segment):
            raise ValueError(f"Constructed path {path_result!r} contains dot-segment {segment!r} which is not allowed")

    result = path_result
    if query_template is not None:
        result += "?" + _interpolate(query_template, kwargs, _quote_query_part)
    if fragment_template is not None:
        result += "#" + _interpolate(fragment_template, kwargs, _quote_fragment_part)

    return result



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .. import _legacy_response
from ..types import journal_entry_list_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .._base_client import make_request_options

__all__ = ["JournalEntries", "AsyncJournalEntries"]


class JournalEntries(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JournalEntriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return JournalEntriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JournalEntriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return JournalEntriesWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Retrieve a specific journal entry

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template("/api/journal_entries/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        journal_report_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Retrieve a list of journal entries

        Args:
          journal_report_id: The ID of the journal report

          page: Page number for pagination

          per_page: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/api/journal_entries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "journal_report_id": journal_report_id,
                        "page": page,
                        "per_page": per_page,
                    },
                    journal_entry_list_params.JournalEntryListParams,
                ),
            ),
            cast_to=NoneType,
        )


class AsyncJournalEntries(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJournalEntriesWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJournalEntriesWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJournalEntriesWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncJournalEntriesWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Retrieve a specific journal entry

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template("/api/journal_entries/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list(
        self,
        *,
        journal_report_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Retrieve a list of journal entries

        Args:
          journal_report_id: The ID of the journal report

          page: Page number for pagination

          per_page: Number of items per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/api/journal_entries",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "journal_report_id": journal_report_id,
                        "page": page,
                        "per_page": per_page,
                    },
                    journal_entry_list_params.JournalEntryListParams,
                ),
            ),
            cast_to=NoneType,
        )


class JournalEntriesWithRawResponse:
    def __init__(self, journal_entries: JournalEntries) -> None:
        self._journal_entries = journal_entries

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            journal_entries.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            journal_entries.list,
        )


class AsyncJournalEntriesWithRawResponse:
    def __init__(self, journal_entries: AsyncJournalEntries) -> None:
        self._journal_entries = journal_entries

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            journal_entries.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            journal_entries.list,
        )


class JournalEntriesWithStreamingResponse:
    def __init__(self, journal_entries: JournalEntries) -> None:
        self._journal_entries = journal_entries

        self.retrieve = to_streamed_response_wrapper(
            journal_entries.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            journal_entries.list,
        )


class AsyncJournalEntriesWithStreamingResponse:
    def __init__(self, journal_entries: AsyncJournalEntries) -> None:
        self._journal_entries = journal_entries

        self.retrieve = async_to_streamed_response_wrapper(
            journal_entries.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            journal_entries.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["InternalAccountUpdateAccountCapabilityParams"]


class InternalAccountUpdateAccountCapabilityParams(TypedDict, total=False):
    internal_account_id: Required[str]

    identifier: Required[str]
    """
    A unique reference assigned by your bank for tracking and recognizing payment
    files. It is important this is formatted exactly how the bank assigned it.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .ledger_balance import LedgerBalance

__all__ = ["LedgerBalances"]


class LedgerBalances(BaseModel):
    available_balance: LedgerBalance
    """
    The available_balance is the sum of all posted inbound entries and pending
    outbound entries. For credit normal, available_amount = posted_credits -
    pending_debits; for debit normal, available_amount = posted_debits -
    pending_credits.
    """

    pending_balance: LedgerBalance
    """The pending_balance is the sum of all pending and posted entries."""

    posted_balance: LedgerBalance
    """The posted_balance is the sum of all posted entries."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ContactDetailCreateRequestParam"]


class ContactDetailCreateRequestParam(TypedDict, total=False):
    contact_identifier: str

    contact_identifier_type: Literal["email", "phone_number", "website"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["Address"]


class Address(BaseModel):
    id: str

    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    created_at: datetime

    line1: Optional[str] = None

    line2: Optional[str] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    locality: Optional[str] = None
    """Locality or City."""

    object: str

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

import builtins
from typing import TYPE_CHECKING, Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.transaction_direction import TransactionDirection

__all__ = ["InternalAccountUpdateAccountCapabilityResponse"]


class InternalAccountUpdateAccountCapabilityResponse(BaseModel):
    id: str

    created_at: datetime

    direction: TransactionDirection
    """One of `debit` or `credit`.

    Indicates the direction of money movement this capability is responsible for.
    """

    discarded_at: Optional[datetime] = None

    identifier: Optional[str] = None
    """
    A unique reference assigned by your bank for tracking and recognizing payment
    files. It is important this is formatted exactly how the bank assigned it.
    """

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    payment_type: Literal[
        "ach",
        "au_becs",
        "bacs",
        "base",
        "book",
        "card",
        "chats",
        "check",
        "cross_border",
        "dk_nets",
        "eft",
        "ethereum",
        "gb_fps",
        "hu_ics",
        "interac",
        "masav",
        "mx_ccen",
        "neft",
        "nics",
        "nz_becs",
        "pl_elixir",
        "polygon",
        "provxchange",
        "ro_sent",
        "rtp",
        "se_bankgirot",
        "sen",
        "sepa",
        "sg_giro",
        "sic",
        "signet",
        "sknbi",
        "solana",
        "stablecoin",
        "wire",
        "zengin",
    ]
    """
    Indicates the the type of payment this capability is responsible for
    originating.
    """

    updated_at: datetime

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, builtins.object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> builtins.object: ...
    else:
        __pydantic_extra__: Dict[str, builtins.object]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import date, datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from .ledger_entry_create_request import LedgerEntryCreateRequest

__all__ = ["LedgerTransactionCreateRequest"]


class LedgerTransactionCreateRequest(TypedDict, total=False):
    ledger_entries: Required[Iterable[LedgerEntryCreateRequest]]
    """An array of ledger entry objects."""

    description: Optional[str]
    """An optional description for internal use."""

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    The timestamp (ISO8601 format) at which the ledger transaction happened for
    reporting purposes.
    """

    effective_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    The date (YYYY-MM-DD) on which the ledger transaction happened for reporting
    purposes.
    """

    external_id: str
    """A unique string to represent the ledger transaction.

    Only one pending or posted ledger transaction may have this ID in the ledger.
    """

    ledgerable_id: str
    """
    If the ledger transaction can be reconciled to another object in Modern
    Treasury, the id will be populated here, otherwise null.
    """

    ledgerable_type: Literal["expected_payment", "incoming_payment_detail", "payment_order", "return", "reversal"]
    """
    If the ledger transaction can be reconciled to another object in Modern
    Treasury, the type will be populated here, otherwise null. This can be one of
    payment_order, incoming_payment_detail, expected_payment, return, or reversal.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """

    status: Literal["archived", "pending", "posted"]
    """To post a ledger transaction at creation, use `posted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["AddressRequest"]


class AddressRequest(BaseModel):
    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Optional[str] = None

    line2: Optional[str] = None

    locality: Optional[str] = None
    """Locality or City."""

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["LegalEntityAssociation"]


class LegalEntityAssociation(BaseModel):
    id: str

    child_legal_entity: "ChildLegalEntity"
    """The child legal entity."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    ownership_percentage: Optional[int] = None
    """The child entity's ownership percentage iff they are a beneficial owner."""

    parent_legal_entity_id: str
    """The ID of the parent legal entity.

    This must be a business or joint legal entity.
    """

    relationship_types: List[Literal["authorized_signer", "beneficial_owner", "control_person"]]

    title: Optional[str] = None
    """The job title of the child entity at the parent entity."""

    updated_at: datetime


from .child_legal_entity import ChildLegalEntity



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["LegalEntityAddressCreateRequest"]


class LegalEntityAddressCreateRequest(BaseModel):
    country: Optional[str] = None
    """Country code conforms to [ISO 3166-1 alpha-2]"""

    line1: Optional[str] = None

    locality: Optional[str] = None
    """Locality or City."""

    postal_code: Optional[str] = None
    """The postal code of the address."""

    region: Optional[str] = None
    """Region or State."""

    address_types: Optional[
        List[Literal["business", "business_registered", "mailing", "other", "po_box", "residential"]]
    ] = None
    """The types of this address."""

    line2: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from .. import _legacy_response
from ..types import account_detail_list_params, account_detail_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ..pagination import SyncPage, AsyncPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.account_detail import AccountDetail
from ..types.shared.accounts_type import AccountsType

__all__ = ["AccountDetails", "AsyncAccountDetails"]


class AccountDetails(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountDetailsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AccountDetailsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountDetailsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AccountDetailsWithStreamingResponse(self)

    def create(
        self,
        account_id: str,
        *,
        accounts_type: Literal["external_accounts"],
        account_number: str,
        account_number_type: Literal[
            "au_number",
            "base_address",
            "card_token",
            "clabe",
            "ethereum_address",
            "hk_number",
            "iban",
            "id_number",
            "nz_number",
            "other",
            "pan",
            "polygon_address",
            "sg_number",
            "solana_address",
            "wallet_address",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> AccountDetail:
        """
        Create an account detail for an external account.

        Args:
          account_number: The account number for the bank account.

          account_number_type: One of `iban`, `clabe`, `wallet_address`, or `other`. Use `other` if the bank
              account number is in a generic format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details", accounts_type=accounts_type, account_id=account_id
            ),
            body=maybe_transform(
                {
                    "account_number": account_number,
                    "account_number_type": account_number_type,
                },
                account_detail_create_params.AccountDetailCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountDetail,
        )

    def retrieve(
        self,
        id: str,
        *,
        accounts_type: AccountsType,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountDetail:
        """
        Get a single account detail for a single internal or external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details/{id}",
                accounts_type=accounts_type,
                account_id=account_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountDetail,
        )

    def list(
        self,
        account_id: str,
        *,
        accounts_type: AccountsType,
        after_cursor: Optional[str] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPage[AccountDetail]:
        """
        Get a list of account details for a single internal or external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details", accounts_type=accounts_type, account_id=account_id
            ),
            page=SyncPage[AccountDetail],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "per_page": per_page,
                    },
                    account_detail_list_params.AccountDetailListParams,
                ),
            ),
            model=AccountDetail,
        )

    def delete(
        self,
        id: str,
        *,
        accounts_type: Literal["external_accounts"],
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Delete a single account detail for an external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details/{id}",
                accounts_type=accounts_type,
                account_id=account_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class AsyncAccountDetails(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountDetailsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountDetailsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountDetailsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Modern-Treasury/modern-treasury-python#with_streaming_response
        """
        return AsyncAccountDetailsWithStreamingResponse(self)

    async def create(
        self,
        account_id: str,
        *,
        accounts_type: Literal["external_accounts"],
        account_number: str,
        account_number_type: Literal[
            "au_number",
            "base_address",
            "card_token",
            "clabe",
            "ethereum_address",
            "hk_number",
            "iban",
            "id_number",
            "nz_number",
            "other",
            "pan",
            "polygon_address",
            "sg_number",
            "solana_address",
            "wallet_address",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> AccountDetail:
        """
        Create an account detail for an external account.

        Args:
          account_number: The account number for the bank account.

          account_number_type: One of `iban`, `clabe`, `wallet_address`, or `other`. Use `other` if the bank
              account number is in a generic format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details", accounts_type=accounts_type, account_id=account_id
            ),
            body=await async_maybe_transform(
                {
                    "account_number": account_number,
                    "account_number_type": account_number_type,
                },
                account_detail_create_params.AccountDetailCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=AccountDetail,
        )

    async def retrieve(
        self,
        id: str,
        *,
        accounts_type: AccountsType,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountDetail:
        """
        Get a single account detail for a single internal or external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details/{id}",
                accounts_type=accounts_type,
                account_id=account_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountDetail,
        )

    def list(
        self,
        account_id: str,
        *,
        accounts_type: AccountsType,
        after_cursor: Optional[str] | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AccountDetail, AsyncPage[AccountDetail]]:
        """
        Get a list of account details for a single internal or external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details", accounts_type=accounts_type, account_id=account_id
            ),
            page=AsyncPage[AccountDetail],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after_cursor": after_cursor,
                        "per_page": per_page,
                    },
                    account_detail_list_params.AccountDetailListParams,
                ),
            ),
            model=AccountDetail,
        )

    async def delete(
        self,
        id: str,
        *,
        accounts_type: Literal["external_accounts"],
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
        idempotency_key: str | None = None,
    ) -> None:
        """
        Delete a single account detail for an external account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds

          idempotency_key: Specify a custom idempotency key for this request
        """
        if not accounts_type:
            raise ValueError(f"Expected a non-empty value for `accounts_type` but received {accounts_type!r}")
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/api/{accounts_type}/{account_id}/account_details/{id}",
                accounts_type=accounts_type,
                account_id=account_id,
                id=id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                idempotency_key=idempotency_key,
            ),
            cast_to=NoneType,
        )


class AccountDetailsWithRawResponse:
    def __init__(self, account_details: AccountDetails) -> None:
        self._account_details = account_details

        self.create = _legacy_response.to_raw_response_wrapper(
            account_details.create,
        )
        self.retrieve = _legacy_response.to_raw_response_wrapper(
            account_details.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            account_details.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            account_details.delete,
        )


class AsyncAccountDetailsWithRawResponse:
    def __init__(self, account_details: AsyncAccountDetails) -> None:
        self._account_details = account_details

        self.create = _legacy_response.async_to_raw_response_wrapper(
            account_details.create,
        )
        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            account_details.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            account_details.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            account_details.delete,
        )


class AccountDetailsWithStreamingResponse:
    def __init__(self, account_details: AccountDetails) -> None:
        self._account_details = account_details

        self.create = to_streamed_response_wrapper(
            account_details.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            account_details.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            account_details.list,
        )
        self.delete = to_streamed_response_wrapper(
            account_details.delete,
        )


class AsyncAccountDetailsWithStreamingResponse:
    def __init__(self, account_details: AsyncAccountDetails) -> None:
        self._account_details = account_details

        self.create = async_to_streamed_response_wrapper(
            account_details.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            account_details.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            account_details.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            account_details.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["LedgerTransactionCreatePartialPostParams", "PostedLedgerEntries", "PostedLedgerEntry"]


class LedgerTransactionCreatePartialPostParams(TypedDict, total=False):
    posted_ledger_entries: Required[Iterable[PostedLedgerEntry]]
    """An array of ledger entry objects to be set on the posted ledger transaction.

    There must be one entry for each of the existing entries with a lesser amount
    than the existing entry.
    """

    description: str
    """An optional free-form description for the posted ledger transaction.

    Maximum of 1000 characters allowed.
    """

    effective_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    The timestamp (IS08601 format) at which the posted ledger transaction happened
    for reporting purposes.
    """

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """


class PostedLedgerEntry(TypedDict, total=False):
    amount: Required[int]
    """Value in specified currency's smallest unit.

    e.g. $10 would be represented as 1000. Can be any integer up to 36 digits.
    """

    direction: Required[Literal["credit", "debit"]]
    """One of `credit`, `debit`.

    Describes the direction money is flowing in the transaction. A `credit` moves
    money from your account to someone else's. A `debit` pulls money from someone
    else's account to your own. Note that wire, rtp, and check payments will always
    be `credit`.
    """

    ledger_account_id: Required[str]
    """The ledger account that this ledger entry is associated with."""

    metadata: Dict[str, str]
    """Additional data represented as key-value pairs.

    Both the key and value must be strings.
    """


PostedLedgerEntries = PostedLedgerEntry
"""This type is deprecated and will be removed in a future release.

Please use PostedLedgerEntry instead.
"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .address import Address as Address
from .currency import Currency as Currency
from .accounts_type import AccountsType as AccountsType
from .async_response import AsyncResponse as AsyncResponse
from .contact_detail import ContactDetail as ContactDetail
from .ledger_balance import LedgerBalance as LedgerBalance
from .address_request import AddressRequest as AddressRequest
from .ledger_balances import LedgerBalances as LedgerBalances
from .foreign_exchange_rate import ForeignExchangeRate as ForeignExchangeRate
from .transaction_direction import TransactionDirection as TransactionDirection
from .third_party_verification import ThirdPartyVerification as ThirdPartyVerification
from .child_legal_entity_create import ChildLegalEntityCreate as ChildLegalEntityCreate
from .ledger_entry_create_request import LedgerEntryCreateRequest as LedgerEntryCreateRequest
from .identification_create_request import IdentificationCreateRequest as IdentificationCreateRequest
from .ledger_account_create_request import LedgerAccountCreateRequest as LedgerAccountCreateRequest
from .ledger_transaction_create_request import LedgerTransactionCreateRequest as LedgerTransactionCreateRequest
from .legal_entity_address_create_request import LegalEntityAddressCreateRequest as LegalEntityAddressCreateRequest
from .legal_entity_industry_classification import LegalEntityIndustryClassification as LegalEntityIndustryClassification
from .legal_entity_association_inline_create import (
    LegalEntityAssociationInlineCreate as LegalEntityAssociationInlineCreate,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .shared.address import Address

__all__ = ["RoutingDetail"]


class RoutingDetail(BaseModel):
    id: str

    bank_address: Optional[Address] = None

    bank_name: str
    """The name of the bank."""

    created_at: datetime

    discarded_at: Optional[datetime] = None

    live_mode: bool
    """
    This field will be true if this object exists in the live environment or false
    if it exists in the test environment.
    """

    object: str

    payment_type: Optional[
        Literal[
            "ach",
            "au_becs",
            "bacs",
            "base",
            "book",
            "card",
            "chats",
            "check",
            "cross_border",
            "dk_nets",
            "eft",
            "ethereum",
            "gb_fps",
            "hu_ics",
            "interac",
            "masav",
            "mx_ccen",
            "neft",
            "nics",
            "nz_becs",
            "pl_elixir",
            "polygon",
            "provxchange",
            "ro_sent",
            "rtp",
            "se_bankgirot",
            "sen",
            "sepa",
            "sg_giro",
            "sic",
            "signet",
            "sknbi",
            "solana",
            "stablecoin",
            "wire",
            "zengin",
        ]
    ] = None
    """
    If the routing detail is to be used for a specific payment type this field will
    be populated, otherwise null.
    """

    routing_number: str
    """The routing number of the bank."""

    routing_number_type: Literal[
        "aba",
        "au_bsb",
        "br_codigo",
        "ca_cpa",
        "chips",
        "cnaps",
        "dk_interbank_clearing_code",
        "gb_sort_code",
        "hk_interbank_clearing_code",
        "hu_interbank_clearing_code",
        "id_sknbi_code",
        "il_bank_code",
        "in_ifsc",
        "jp_zengin_code",
        "mx_bank_identifier",
        "my_branch_code",
        "nz_national_clearing_code",
        "pl_national_clearing_code",
        "se_bankgiro_clearing_code",
        "sg_interbank_clearing_code",
        "swift",
        "za_national_clearing_code",
    ]
    """The type of routing number.

    See https://docs.moderntreasury.com/platform/reference/routing-detail-object for
    more details.
    """

    updated_at: datetime
